# -*- coding: utf-8 -*-
"""
Provides classes for parsing exported data of Hatena diary.

This module has two classes.
One is support for Movable Type format,
the other is support XML a.k.a. Hatena Diary format.

Developed for MovableType format, but it has many bugs of convering
hyperlink and footnote and more. Then rewriten for XML format against
squashing these bugs. So recommend using for XML format.

-------------------------------------------------------------------------------
Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import os.path
import utils
import htnparser
import commands
from __init__ import __dstdir__


def xml2rest(infile, dstdir=None, retrieve_image_flag=False):

    if dstdir is None:
        dstdir = os.path.expanduser(__dstdir__)
    else:
        dstdir = os.patn.expanduser(dstdir)

    # make directory
    mkdir(dstdir)

    if not os.path.isfile(dstdir + 'conf.py'):
        # chdir to dstdir
        os.chdir(dstdir)

        # exec tinker setup
        commands.getstatusoutput('tinker -s')

    p = htnparser.HatenaXMLParser(infile, dstdir, retrieve_image_flag)

    path_str = '\n'
    for d in p.list_day_element():
        dirpath, bodies, comments = p.handle_entries_per_day(d)

        # make directory
        mkdir(dstdir + dirpath)

        for body in bodies[::-1]:

            timestamp = body[1]
            title = body[0]
            categories = body[2]
            body = body[3]

            # append path for master.rst
            path_str = '   ' + dirpath + timestamp + '\n' + path_str

            with open(dstdir + dirpath + timestamp + '.rst', 'w') as f:
                f.write(output(title, categories, body).encode('utf-8'))

            if comments:
                for comment in comments:
                    ''' '''

    with open(dstdir + 'master.rst', 'a') as f:
        f.write(path_str)


def mkdir(dirpath):
    if not os.path.isdir(dirpath):
        os.makedirs(dirpath)


def output(title, categories, body):
    body_str = (title + '\n' +
                '=' * utils.length_str(title) + '\n\n' +
                body + '\n\n' + footer(categories))
    return body_str


def footer(categories):
    pat_list = re.compile('\[|\]')
    footer_str = ('.. author:: default\n' +
            '.. categories:: ' + category(categories) + '\n' +
            '.. tags::\n' +
            '.. comments::\n')
    return footer_str


def category(categories=None):
    if categories is None:
        return ''
    cat_str = ''
    for cat in categories:
        cat_str += cat + ', '
    return cat_str



# -*- coding: utf-8 -*-
"""
Provides classes for parsing exported data of Hatena diary.

This module has two classes.
One is support for Movable Type format,
the other is support XML a.k.a. Hatena Diary format.

Developed for MovableType format, but it has many bugs of convering
hyperlink and footnote and more. Then rewriten for XML format against
squashing these bugs. So recommend using for XML format.

-------------------------------------------------------------------------------
Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
from HTMLParser import HTMLParser
import xml.etree.ElementTree
import utils
import convert
from __init__ import __imgdir__


class HatenaXMLParser(object):
    """Hatena specify XML format."""

    def __init__(self, file, dstdir, retrieve_image_flag=False):
        """Read exported XML file, and initialize parameters.

        Argument:

            file: exported file
        """
        with open(file) as f:
            self.xmlobj = xml.etree.ElementTree.ElementTree(file=f)
            self.code_flag = False
            self.table_flag = False
            self.list_lv = 0
            self.ref_flag = False
        self.retrieve_image_flag = retrieve_image_flag
        self.dstdir = dstdir

    def list_day_element(self):
        """Day element is daily unit of Hatena diary exported data,
        it has one blog entry or two or more entries.
        So handle per day element."""
        return self.xmlobj.getroot().findall('day')

    def handle_entries_per_day(self, day_element):
        """Get element of day, and process conversion

        Argument:

            day_element: day element of XML.
        """
        date = day_element.get('date')
        body_element = day_element.find('body')
        comments_element = day_element.find('comments')

        # make directory when it is not existed.
        dirpath = (date.replace('-', '/') + '/')

        # write entry to reST file.
        bodies = self.handle_body_elements(body_element)

        comments = None
        # write comments to reST file.
        if comments_element is not None:
            comments = convert.handle_comments_element(comments_element)

        return dirpath, bodies, comments

    def handle_body_elements(self, body_element):
        """Separate data why body element has multi entry of diary.

        Starting is 1, not also 0.

        Argument:

            bodies: text string of body element from exported XML data.
        """

        # remove multiple lines of html comment.
        prog = re.compile('\n\<!--(.*\n)*--\>')
        bodies_text = re.sub(prog, '', body_element.text)

        """Separate text of multi entries with newline(\n),
        but excepting below cases;

        shell script switch default: \n\*(?!\))
        section, subsection: \n\*(?!\*)
        listing of reST: \n\*(?!\ )
        continuous line feed: \n(?!\n)
        crontab format: \n\*(?!/\d?\ \*)
        """
        prog2 = re.compile('\n\*(?!\)|\*|\ |\n|/\d?\ \*)')
        list_entries = prog2.split(bodies_text)

        # main loop
        # TODO: move to main() later.
        bodies = [self.handle_body(body_text)
                  for body_text in list_entries
                  if body_text]
        return bodies

    def handle_body(self, body_text):
        """Handle body.

        Argument:

            body_text: text string of one blog entry.
        """
        if body_text:
            timestamp, categories, title = convert.get_metadata(
                body_text.split('\n', 1)[0])
            entry_body = convert.extract_entry_body(body_text)
            rested_body = self.hatena2rest(entry_body)

            return title, timestamp, categories, rested_body

    def hatena2rest(self, str_body):
        """Convert body text of day entry to rest format.

        Argument:

            string: convert target string.
        """
        footnotes = ''
        table = []
        tables = []
        merge_string = ''

        def parse_begin_ref(string_line):
            """Parse begining of reference block

            Argument:

                string: convert target string.
            """
            pat_start_ref, match_obj = utils.regex_search(
                '^>((http|https)://(.+?)|)>$', string_line)
            if match_obj:
                self.ref_flag = True
                if match_obj.group(1):
                    repl_str = match_obj.group(1)
                else:
                    repl_str = ''

                string_line = pat_start_ref.sub(
                    repl_str,
                    string_line)

            return string_line

        def parse_end_ref(string_line):
            """Parse ending of reference block

            Argument:

                string: convert target string.
            """
            pat_end_ref, match_obj = utils.regex_search(
                '^<<', string_line)
            if match_obj:
                string_line = pat_end_ref.sub('\n\n', string_line)
                self.ref_flag = False
            else:
                string_line = re.sub('^', '   ', string_line)
            return string_line

        def parse_begin_codeblock(string_line):
            """Parse begining of code block

            Argument:

                string: convert target string.
            """
            pat_code_open, match_obj = utils.regex_search(
                '>\|([a-zA-Z0-9]*)\|$|>\|()$', string_line)
            if match_obj:
                # code block opening
                self.code_flag = True
                if match_obj.group(1):
                    lexer_str = convert.replace_lexer(match_obj.group(1))
                    string_line = pat_code_open.sub(
                        '\n.. code-block:: ' + lexer_str + '\n',
                        string_line)
                else:
                    string_line = pat_code_open.sub(
                        '\n.. code-block:: sh\n', string_line)
            return string_line

        def parse_end_codeblock(string_line):
            """Parse ending of codeblock.

            Argument:

                string_line: parsing target string.
            """
            pat_code_close, match_obj = utils.regex_search(
                '^\|\|<|^\|<$', string_line)
            if match_obj:
                string_line = pat_code_close.sub('\n', string_line)
                # code block closing
                self.code_flag = False
            else:
                string_line = re.sub('^', '   ', string_line)
            return string_line

        def extract_tables(string_line, table, tables):
            """Extract tables

            Argument:

                string_line: parsing target string.
                table:       parsing target table
                tables:      parsing target tables
            """
            pat_table, match_obj = utils.regex_search(
                '^\|(.+?)\|$', string_line)
            if match_obj:
                row_data = (match_obj.group(0),
                            match_obj.groups()[0].split('|'))
                if not self.table_flag:
                    # table start
                    self.table_flag = True
                table.append(row_data)
            else:
                if self.table_flag:
                    # table close
                    tables.append(table)
                    table = []
                    self.table_flag = False
            return table, tables

        if str_body:
            # str_line exclude '\n'
            for str_line in str_body.split('\n'):

                # convert hyperlink
                str_line = convert.convert_hyperlink(str_line)

                # handle line inside code block
                if self.code_flag:
                    str_line = parse_end_codeblock(str_line)

                # handle line outside code block
                else:
                    str_line = parse_begin_codeblock(str_line)

                    # replace '*' to '\*' of inline
                    str_line = convert.replace_asterisk(str_line)

                    # listing
                    str_line = self.listing2rest(str_line)

                    # convert shell var
                    str_line = convert.replace_shell_variable(str_line)

                    # section , subsection
                    str_line = convert.section2rest(str_line)

                    # convert image from hatena fotolife
                    str_line = self.fotolife2rest(str_line)

                    # convert footnote
                    str_line, footnotes_ = convert.footnote2rest(str_line)
                    if footnotes_:
                        footnotes += footnotes_ + '\n'

                    # convert refs
                    if self.ref_flag:
                        str_line = parse_end_ref(str_line)
                    else:
                        str_line = parse_begin_ref(str_line)

                    # extract table data
                    table, tables = extract_tables(str_line, table, tables)

                    # remove internal_link and convert blog parts
                    str_line = self.convert_blog_parts(str_line)

                merge_string += utils.remove_element_entity(str_line) + '\n'

            # convert table
            merge_string = self.table2rest(tables, merge_string)
            self.code_flag = False
            return merge_string + '\n' + footnotes

    def table2rest(self, tables, merge_string):
        """Convert hatena syntax to reST of table

            Argument:

                tables:       list of table
                merge_string: convertd and concatnated strings
        """

        # replace table
        for table in tables:
            """
            tables is list: [table, table, ...]
            table is list:  [row, row]
            """
            replace_line = ''
            border = ''

            columns_width = [0] * len(table[1][1])

            # get columns width
            columns_width = convert.get_columns_width_list(
                table, columns_width)

            # columns_width has max values when this step
            for row_i, row in enumerate(table):

                row_str = ''
                # get row string, row border
                row_str, border = convert.convert_row(
                    row, row_i, columns_width, row_str, border)

                # merge row string with row
                merge_row_str = convert.merge_row_string(row_str, border)

                # merge string with row string
                if row_i == len(table) - 1:
                    merge_string = merge_string.replace(
                        row[0] + '\n', merge_row_str + '\n' + border, 1)
                else:
                    merge_string = merge_string.replace(
                        row[0] + '\n', merge_row_str, 1)

        return merge_string

    def fotolife2rest(self, str_line):
        """Convert fotolife to image directive.

        Argument:

            str_line: text string of blog entry.

        convert is
            from: hatena; [f:id:imageid:image]
            to:   reST  ; .. image:: imgsrc
                              :target: uri
        """
        r, m = utils.regex_search(
            '\[f:id:(.*):([0-9]*)[a-z]:image(|:.+?)\]', str_line)
        if m:
            img_uri_partial = ('http://cdn-ak.f.st-hatena.com/images/fotolife/'
                               + m.group(1)[0] + '/' + m.group(1) + '/'
                               + m.group(2)[0:8] + '/' + m.group(2))
            # get image file
            img_src = utils.retrieve_image(img_uri_partial,
                                           self.dstdir + __imgdir__,
                                           self.retrieve_image_flag)
            repl_str = ('\n.. image:: ' + __imgdir__ + img_src)
            str_line = r.sub(repl_str, str_line)
        return str_line

    def listing2rest(self, str_line):
        """Convert hatena syntax to reST of list.

        Argument:

            str_line: text string of blog entry.

        """

        for i in range(1, 4)[::-1]:
            """list lv is indent depth
            order is 3,2,1 why short matche is stronger than long.
            3 is --- or +++
            2 is -- or ++
            1 is - or +
            """
            r, m = utils.regex_search(
                '(^(-{%d}))|(^(\+{%d}))' % (i, i), str_line)
            if m:
                item = ('  ' * (i - 1) + '* ' if m.group(1)
                        else '  ' * (i - 1) + '#. ')
                if self.list_lv == i:
                    repl = item
                else:
                    repl = '\n' + item
                    self.list_lv = i
                str_line = r.sub(repl, str_line)
        str_line += '\n'
        return str_line

    def convert_blog_parts(self, str_line):
        """Convert blog parts to reST.

        Argument:

            str_line: text string of blog entry.

        """

        # remove hatena internal link
        str_line = convert.remove_internal_link(str_line)

        # for ditto
        str_line = convert.ditto(str_line)

        # remove span element
        str_line = convert.remove_span(str_line)

        # remove del element
        str_line = convert.remove_del(str_line)

        # for google maps
        str_line = convert.google_maps(str_line)

        # for gmodules
        str_line = convert.gmodules(str_line)

        # for img element
        str_line = convert.img2image(str_line)

        # for amazlet
        pat_amazon, m = utils.regex_search('amazlet', str_line)
        if not m:
            pat_image, m = utils.regex_search(
                '(<a href="(.+?)" .+?><img src="(.+?)".*?/?></.+?>)', str_line)
            if m:
                img_path = utils.retrieve_image(m.group(3),
                                                self.dstdir + __imgdir__,
                                                self.retrieve_image_flag)
                str_line = pat_image.sub(
                    '\n.. image:: ' + __imgdir__ + img_path + '\n   :target: '
                    + m.group(2) + '\n\n', str_line)

        # for youtube
        str_line = convert.youtube(str_line)

        # for tweet
        str_line = convert.tweet(str_line)

        # for blogparts
        str_line = convert.extract_blog_parts(str_line)

        return str_line

    def comment2rest(self, comment_text):
        """Convert comment text to reST format.

        Argument:

            comment_text: comment text.
        """

        # remove <br>
        pat_br_tag = re.compile('<br?>', flags=re.U)
        converted_comment = pat_br_tag.sub('', comment_text)
        return converted_comment



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re
import utils
import xml.etree.ElementTree


def replace_lexer(key):
    """Return code-block lexer.

    Argument:

        key: code block syntax name
    """

    lexer = {
        'conf': 'apache',
        'erlang': 'erlang',
        'm4': 'make',
        'log': 'ini',
        'lisp': 'scheme',
        'mail': 'ini',
        'rcs': 'diff',
        'dmesg': 'console',
        'strace': 'console',
        'fstab': 'ini',
        'tree': 'ini',
        'grub': 'ini',
        'emacs': 'scheme',
        'telnet': 'ini',
        'fetchmail': 'ini',
        'dot': 'ini',
        'git': 'diff',
        'TeX': 'latex',
        'Makefile': 'makefile',
        'sudoers': 'makefile',
        'crontab': 'sh',
        'dosbatch': 'bat',
        'sed': 'sh',
        'cc': 'c++',
        'mt': 'sh',
        'thml': 'ini',
        'xml': 'text'
        }
    if lexer.get(key):
        return lexer.get(key)
    else:
        return 'sh'


def convert_hyperlink(string):
    """Convert hyperlink.

    Argument:

        string: text string of blog entry.

    convert is
        from: hatena; [url:title:titlestring]
        to:   reST  ;  `titlestring <url>`_
    """

    pat_line_head = re.compile(
        '^(\[((http|https)://.+?):title=(.+?)\])', flags=re.U)
    for i in pat_line_head.findall(string):
        string = string.replace(
            i[0], '`' + i[3] + ' <' + i[1] + '>`_ ')

    pat_inline = re.compile(
            '(\[((http|https)://.+?):title=(.+?)\])', flags=re.U)
    for i in pat_inline.findall(string):
        string = string.replace(
            i[0], ' `' + i[3] + ' <' + i[1] + '>`_ ')

    return string


def replace_asterisk(string):
    """Replace asterisk

    Argument:

        string: text string of blog entry.
    """

    # except table header
    if string.find('|*') < 0:

        string = string.replace('*', '\*')

        if string.find('\*\*\*') == 0:
            string = string.replace('\*\*\*', '***', 1)
        elif string.find('\*\*') == 0:
            string = string.replace('\*\*', '**', 1)
        elif string.find('\*') == 0:
            string = string.replace('\*', '*', 1)

    return string


def replace_shell_variable(string):
    """Replace shell variable

    Argument:

        string: text string of blog entry.
    """

    pat_shell_var, match_obj = utils.regex_search(
        '(\${.+?}[a-zA-Z0-9/_\\\*]+)', string)
    if match_obj:
        string = pat_shell_var.sub(
            ' :command:`' + match_obj.group() + '` ', string)
    return string


def section2rest(string):
    """Convert hatena syntax to reST of section.

    Argument:

        string: text string of blog entry.
    """

    for i in range(2, 4)[::-1]:
        """2:section, 3:subsection"""
        sep = '-' if i == 2 else '^'
        r, m = utils.regex_search('^(\*){%d}(.*)' % i, string)
        if m:
            pat_space = re.compile('^\s+')
            section_str = pat_space.sub('', m.group(2))
            string = r.sub(
                '\n' + section_str + '\n'
                + sep * utils.length_str(section_str) + '\n', string)
    return string


def footnote2rest(string):
    """Convert hatena syntax to reST of footnote.

    Argument:

        string: text string of blog entry.

    convert is
        from: hatena; ((string))
        to:   reST  ; inline is   [#]_
                      footnote is .. [#] string
    """

    str_rest = string
    footnotes = ''
    pat_fn = re.compile('(\(\((.+?)\)\))', flags=re.U)

    for i in pat_fn.findall(string):
        str_rest = str_rest.replace(i[0], ' [#]_ ')
        if len(pat_fn.findall(string)) > 1:
            footnotes += '\n.. [#] ' + i[1]
        else:
            footnotes += '.. [#] ' + convert_hyperlink(i[1])
    return str_rest, footnotes


def extract_categories(str_categories):
    """Get category of entry.

    Argument:

        str_categories: categories of hatena diary syntax as [a][b][c]

    Return:

        list of categories.
    """
    if str_categories:
        pat_category = re.compile('\[(.+?)\]', flags=re.U)
        list_category = pat_category.findall(str_categories)
        return list_category


def extract_entry_body(body_text):
    """Get body of entry.

    Argument:

        body_text: blog entry text.
    """

    if body_text.find('\n'):
        entry_body = body_text.split('\n', 1)[1]
        return entry_body


def handle_comments_element(comments_element):
    """Handle multiple comment within comments element.

    Argument:

        comments_element: comments element of XML.
    """

    comments = [handle_comment_element(comment_element)
                for comment_element in comments_element]
    return comments


def handle_comment_element(comment_element):
    """Handles comment element.

    Argument:

        comment_element: comment element of XML.
    """

    username = comment_element.find('username').text

    unixtime = comment_element.find('timestamp').text
    timestamp = utils.unix2ctime(unixtime)

    comment = comment_element.find('body').text

    return username, timestamp, comment


def parse_amazlet(xmltree):
    """Convert blog parts of amazlet to hyperlink

    Argument:

        xmltree: XML tree object

    """

    anchor_element = xmltree.find('div').find('a')
    img_element = anchor_element.find('img')

    uri = anchor_element.get('href')
    img_uri = img_element.get('src')
    img_alt = img_element.get('alt')
    repl_amazon = ('\n\n`' + img_alt + ' <' + uri + '>`_\n\n')
    return repl_amazon


def parse_twitter(xmltree):
    """Retrieve URI from blog parts of twitter

    Argument:

        xmltree: XML tree object

    """

    uri = [i.get('href') for i in xmltree.find('p')
           if i.get('title')][0]
    return uri


def parse_slideshare(xmltree):
    """Convert blog parts of slideshare to hyperlink

    Argument:

        xmltree: XML tree object

    """

    uri = xmltree.find('strong').find('a').get('href')
    title = xmltree.find('strong').find('a').get('title')
    repl_slideshare = '\n`' + title + ' <' + uri + '>`_\n'
    return repl_slideshare


def parse_heyquiz(xmltree):
    uri = xmltree.get('href')
    img_src = xmltree.find('img').get('src')
    img_alt = xmltree.find('img').get('alt')
    repl_heyquiz = '\n`' + img_alt + ' <' + img_src + '>`_\n'
    return repl_heyquiz


def convert_row(row, row_i, columns_width, row_str, border):
    """Convert row of table.

    Argument:

        row:           taple of row string of hatena syntax and columns values
        row_i:         row index
        columns_width: list of maximum width of each columns
        row_str:       converted row string
        border:        reST table border line string

    """

    for i in range(len(row[1])):
        # numbers of values
        if i < len(row[1]) - 1:
            row_str += ("  " + row[1][i] +
                        " " * (columns_width[i] -
                               utils.length_str(row[1][i])) + ' ')
            if row_i == 0:
                border += (" " + "=" * (columns_width[i] + 2))
        else:
            row_str += ("  " + row[1][i] +
                        " " * (columns_width[i]
                               - utils.length_str(row[1][i]))
                        + '  ')
            if row_i == 0:
                border += (" " + "=" * (columns_width[i] + 2) + ' ')
    return (row_str, border)


def get_columns_width_list(table, columns_width):
    """Retrieve list of maximum width of each columns.

    Argument:

        table:         list of rows
        columns_width: list of maximum width of each columns

    """

    for row in table:
        '''
        row is tuple; (pattern, values)
        row[0] is pattern
        row[1] is values list
        '''
        for i in range(len(row[1])):
            if columns_width[i] <= utils.length_str(row[1][i]):
                columns_width[i] = utils.length_str(row[1][i])
            else:
                columns_width[i] = columns_width[i]
    return columns_width


def merge_row_string(row_str, border):
    """Merge string from each rows

    Argument:

        row_str: row string merged each columns string
        border:  table border line string

    """

    merge_row_str = ''
    pat_row = re.compile(' \*')
    if pat_row.search(row_str):
        row_str = pat_row.sub('  ', row_str)
        merge_row_str += border + '\n' + row_str + '\n' + border
    else:
        merge_row_str += row_str
    return merge_row_str


def remove_span(string):
    """Remove span element.

    Argument:

        string: blog entry body string.

    """

    pat_span, m = utils.regex_search(
        '(<span .+?>(.+?)</span>)', string)
    if m:
        string = pat_span.sub(m.group(2), string)
    return string


def remove_del(string):
    """Remove del element.

    Argument:

        string: blog entry body string.

    """

    pat_del, m = utils.regex_search(
        '(<del( .+?|)>(.+?)</del>)', string)
    if m:
        string = pat_del.sub('', string)
    return string


def img2image(string):
    """Convert html img element to reST image directive.

    Argument:

        string: blog entry body string.

    """

    pat_img, m = utils.regex_search('^<img src="(.+?)" .+?(/?)>', string)
    if m:
        string = pat_img.sub('\n.. image:: ' + m.group(1)
                               + '\n\n', string)
    return string


def google_maps(string):
    """Convert blog parts of google maps to reST raw directive.

    Argument:

        string: blog entry body string.

    """

    if (string.find('http://maps.google.com/') > 0 or
        string.find('http://maps.google.co.jp/') > 0):
        pat_google_maps, m = utils.regex_search(
            '(<iframe .+?></iframe><br />(<.+?>.+?</.+?>)(.*?)</.+?>)',
            string)
        if m:
            string = pat_google_maps.sub(
                '\n.. raw:: html\n\n    ' + m.group(0) + '\n', string)
    return string


def gmodules(string):
    """Convert blog parts of gmodules to reST raw directive.

    Argument:

        string: blog entry body string.

    """

    if (string.find('http://gmodules.com') > 0 or
        string.find('https://gist.github.com') > 0):
        pat_gmodules, m = utils.regex_search(
            '^<script .+?></script>', string)
        if m:
            string = pat_gmodules.sub(
                '\n.. raw:: html\n\n    ' + m.group(0) + '\n', string)
    return string


def youtube(string):
    """Convert blog parts of YouTube to reST raw directive.

    Argument:

        string: blog entry body string.

    """

    if string.find('http://www.youtube.com') > 0:
        pat_youtube, m = utils.regex_search(
            '(<object .+?>(.*?)</.+?>)', string)
        if m:
            string = pat_youtube.sub(m.group(0), string)
            string = string.replace('\n', '')
            string = string.replace('&hl=ja', '')
            string = string.replace('&fs=1', '')
            string = '\n.. raw:: html\n\n   ' + string + '\n'
    return string


def get_metadata(string):
    """Get metadata of entry.

    Argument:

        string: title line string of hatena blog entry.
    """
    if string:
        """pattern a)

        timestamp: (\d*)
        category: (\[.*\])*
        title with uri: (\[http?://.*\])(.*)
        """
        pat_title = re.compile('\*?(\d*)\*(\[.*\])*(\[http?://.*\])(.*)',
                               flags=re.U)

        """pattern b)

        timestamp: (\d*)
        category: (\[.*\])*
        title: (.*)
        """
        pat_title_with_link = re.compile(
            '\*?(\d*)\*(\[.*\])*(.*)', flags=re.U)

        if pat_title.search(string):
            # pattern a)
            timestamp, str_categories, linked_title, str_title = (
                pat_title.search(string).groups())

            title = convert_hyperlink(
                linked_title) + str_title

        elif pat_title_with_link.search(string):
            # pattern b)
            timestamp, str_categories, title = (
                pat_title_with_link.search(string).groups())

        return (utils.unix2ctime(timestamp, date_enabled=False),
                extract_categories(str_categories), title)


def parse_blog_parts(string):
    """Parse and convert blog parts.

    Argument:

        string: blog entry body string.

    """

    ex_ref_char = re.compile('\&(?!amp;)')
    string = ex_ref_char.sub('&amp;', string)

    string = string.replace('alt="no image"', '')

    try:
        xmltree = xml.etree.ElementTree.fromstring(string)
    except:
        utils.error(string)

    if xmltree.get('class') == 'amazlet-box':
        repl_amazon = parse_amazlet(xmltree)
        return repl_amazon
    if xmltree.get('class'):
        if xmltree.get('class').find('bbpBox') == 0:
            repl_twitter = parse_twitter(xmltree)
            return repl_twitter
    if str(xmltree.get('id')).find('__ss_') == 0:
        repl_slideshare = parse_slideshare(xmltree)
        return repl_slideshare
    if str(xmltree.get('href')).find('heyquiz.com') >= 0:
        repl_heyquiz = parse_heyquiz(xmltree)
        return repl_heyquiz


def extract_blog_parts(string):
    """Extract blog parts from blog entry string.

    Argument:

        string: blog entry body string.

    """

    html_tags = re.compile('(^(<.+?>(.+?)</.+?>)$)', flags=re.U)
    m = html_tags.search(string)
    if m:
        repl_str = parse_blog_parts(m.group(0).encode('utf-8'))
        string = html_tags.sub(repl_str, string)
    return string


def tweet(string):
    """Convert blog parts of twitter to reST hyperlink.

    Argument:

        string: blog entry body string.

    """

    pat_comment, m = utils.regex_search(
        '((<!-- (.+?) -->) (<.+?>(.+?)</.+?> )(<!-- (.+) -->))', string)
    if m:
        str_tmp = string.replace(m.group(2), '')
        str_tmp = str_tmp.replace(m.group(6), '')
        pat_style, m2 = utils.regex_search(
            ' <style .+?>(.+?)</style> ', str_tmp)
        if m2:
            str_tmp = str_tmp.replace(m2.group(0), '')
            str_tmp = str_tmp.replace('><', '>\n<')
            str_tmp = str_tmp.replace('> <', '>\n<')
            str_tmp = str_tmp.replace('</span>\n', '')
            pat_tweet = re.compile(
                '((<.+?>(.+?)</.+?>)(.+?)(<.+?>(.+?)</.+?>))')
            m3 = pat_tweet.search(str_tmp)
            if m3:
                pat_anchor = re.compile('<a.+?>')
                tweet_msg = (pat_anchor.sub('', m3.group(3)) +
                             pat_anchor.sub('', m3.group(4))
                             + pat_anchor.sub('', m3.group(5))
                             ).replace('</a>', '')

            if parse_blog_parts(str_tmp.encode('utf-8')):
                uri = parse_blog_parts(str_tmp.encode('utf-8'))
                repl_str = '\n' + uri + '::\n\n   ' + tweet_msg + '\n\n'
            else:
                repl_str = ''
            string = pat_comment.sub(repl_str, string)
    return string


def ditto(string):
    """Convert blog parts of twitter with ditto to reST hyperlink.

    Argument:

        string: blog entry body string.

    """

    pat_ditto, m = utils.regex_search(
        '(<style .+?>.+?</style>)(<div .+?>.+?</div>)', string)
    if m:
        ex_ref_char = re.compile('\&(?!amp;)', flags=re.U)
        string = ex_ref_char.sub('&amp;', m.group(2))

        # get uri
        uri = ''
        xmltree = xml.etree.ElementTree.fromstring(string.encode('utf-8'))
        for p_child in xmltree.find('p').getchildren():
            for i, p_child_child in enumerate(p_child.getchildren()):
                if i == 1 and p_child_child.get('href'):
                    uri = p_child_child.get('href')

        # get tweet message
        tweet_msg = ''
        if xmltree.get('class').find('ditto') == 0:
            span_element = xmltree.find('p').find('span').find('span')
            for i, v in enumerate(xmltree.itertext()):
                if i > 1:
                    pat = re.compile('&nbsp;|via', flags=re.U)
                    if pat.search(v) > 0:
                        break
                    else:
                        tweet_msg += str(v.encode('utf-8'))
        repl_str = '\n' + uri + '::\n\n   ' + tweet_msg + '\n\n'
        string = pat_ditto.sub(m.group(), repl_str).decode('utf-8')

    return string


def remove_internal_link(string):
    """Remove hatena internal link.

    Argument:

        string: blog entry body string.

    """

    pat_hatena_internal_link, m = utils.regex_search(
        '(\[\[|\]\])', string)
    if m:
        string = pat_hatena_internal_link.sub('', string)
    return string



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os.path
import sys
import argparse
import processing
import utils
from __init__ import __version__


def parse_options():
    """Parse command line options"""
    prs = argparse.ArgumentParser(description='usage')
    prs.add_argument('-v', '--version', action='version', version=__version__)
    setoption(prs, 'retrieve')
    setoption(prs, 'infile')
    setoption(prs, 'dstdir')
    args = prs.parse_args()
    return args


def setoption(obj, kword):
    """Set option by keyword

    Argument:

        obj:   Parser object
        kword: Keyword of option
    """

    if kword == 'infile':
        obj.add_argument('infile', action='store',
                         help='specify input exported file of hatena diary')

    if kword == 'dstdir':
        obj.add_argument('-d', '--dstdir', action='store',
                         help='specify output destination directory path')

    if kword == 'retrieve':
        obj.add_argument('-r', '--retrieve', action='store_true',
                         help='retrieve image from web services')


def main():
    try:

        args = parse_options()
        f = args.__dict__.get('infile')
        if f.find('~') == 0:
            infile = os.path.expanduser(f)
        else:
            infile = os.path.abspath(f)

        if args.__dict__.get('dstdir'):
            dstdir = args.__dict__.get('dstdir')
        else:
            # default: ~/tmp/hatena2rest/
            dstdir = None

        if args.__dict__.get('retrieve'):
            retrieve_image_flag = True
        else:
            retrieve_image_flag = False

        processing.xml2rest(infile, dstdir, retrieve_image_flag)

    except RuntimeError as e:
        utils.error(e)
        return
    except UnboundLocalError as e:
        utils.error(e)
        return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests of utils.py
"""
import unittest
import time
import hatena2rest.utils as u


class UtilsTests(unittest.TestCase):

    def setUp(self):
        self.unixtime = '2147483647'
        self.hankaku = u'abcABC123-%$s'
        self.zenkaku = u'日本語テスト'
        self.zenhankaku = u'日本語hogeほげ'
        self.html_str = '<del>100</del>もげ&nbsp;200&nbsp; 300\
 <span style="test">400</span>'
        self.html_str2 = '<span><del>100</del>もげ&nbsp;200&nbsp; 300\
 <span style="test">400</span></span>'
        self.regex = '<span(.+?|)>(.+?)</span>'

    def test_unix2ctime(self):
        if time.tzname == 'JST':
            self.assertEqual(u.unix2ctime(self.unixtime, date_enabled=False),
                             '121407')
            self.assertEqual(u.unix2ctime(self.unixtime),
                             'Tue Jan 19 12:14:07 2038')
        if time.timezone == 0:
            self.assertEqual(u.unix2ctime(self.unixtime, date_enabled=False),
                             '031407')
            self.assertEqual(u.unix2ctime(self.unixtime),
                             'Tue Jan 19 03:14:07 2038')

    def test_length_str(self):
        self.assertEqual(u.length_str(self.hankaku), 13)
        self.assertEqual(u.length_str(self.zenkaku), 12)
        self.assertEqual(u.length_str(self.zenhankaku), 14)

    def test_remove_element_entity(self):
        self.assertEqual(u.remove_element_entity(self.html_str),
                         'もげ&nbsp;200&nbsp; 300 400')
        self.assertEqual(u.remove_element_entity(self.html_str2),
                         '<span>もげ&nbsp;200&nbsp; 300 400')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Provides utility function.

This module provides function as common utilities.

-------------------------------------------------------------------------------
Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import re
import os
import glob
import time
import unicodedata
from datetime import datetime
if sys.version_info > (2, 6) and sys.version_info < (2, 8):
    import urllib2 as urllib
elif sys.version_info > (3, 0):
    import urllib.request as urllib
from __init__ import __sleep__
from __init__ import __timeout__
import socket


def unix2ctime(unixtime, date_enabled=True):
    """Get timestamp of entry.

    Argument:

        unixtime: unixtime string
        date_enabled: 'Mon Apr  2 01:00:44 2012' when True
                      (default False '010044')
    """
    if unixtime:
        if date_enabled:
            # for comment
            timestamp = time.ctime(int(unixtime))
        else:
            # for blog entry.
            # reST file name becomes this value + '.rst'.
            prog = re.compile('\s+')
            t = prog.split(time.ctime(int(unixtime)))[3]
            timestamp = t.replace(':', '')
        return timestamp


def length_str(string):
    """calculate Hankaku(1 byte) and Zenkaku(2 byte) summarize length.

    Argument:

        string: calculate target string.

    Return:

        calculated length.
    """
    fwa = ['F', 'W', 'A']
    hnna = ['H', 'N', 'Na']

    if isinstance(string, unicode):
        zenkaku = len([unicodedata.east_asian_width(c)
                       for c in string
                       if unicodedata.east_asian_width(c) in fwa])
        hankaku = len([unicodedata.east_asian_width(c)
                       for c in string
                       if unicodedata.east_asian_width(c) in hnna])
        return (zenkaku * 2 + hankaku)
    elif isinstance(string, str):
        return len(string)


def remove_element_entity(string):

    # remove del element
    pat_del = re.compile('<del>.+?</del>')
    m = pat_del.search(string)
    if m:
        string = pat_del.sub('', string)

    # remove span_begin element
    pat_span_begin = re.compile('<span style=(.+?)>')
    m = pat_span_begin.search(string)
    if m:
        string = pat_span_begin.sub('', string)

    # remove span_end element
    string = string.replace('</span>', '')

    return string


def retrieve_image(img_uri, img_src_dir, retrieve_image_flag=False):

    def check_local_img(img_path):
        return os.path.isfile(img_path)

    def save_image(obj, req, img_path):

        res = obj.open(req)
        data = res.read()

        with open(img_path, 'w') as f:
            f.write(data)
            error('OK: ' + img_path)
            return True

    if not os.path.isdir(img_src_dir):
        os.makedirs(img_src_dir)

    # return image file path
    os.chdir(img_src_dir)
    if glob.glob(os.path.basename(img_uri + '*')):
        img_path = glob.glob(os.path.basename(img_uri + '*'))[0]
        if check_local_img(img_path):
            return img_path

    obj = urllib.build_opener(urllib.HTTPHandler)

    if img_uri.find('fotolife') > 0:

        # for hatena fotolife
        '''
        if check_local_img(img_path):
            return os.path.basename(img_path)
            '''

        for suffix in ('.jpg', '.png'):

            # return image file path
            get_img_path = (lambda img_src_dir, img_uri, suffix:
                                img_src_dir + os.path.basename(img_uri)
                            + suffix)
            img_path = get_img_path(img_src_dir, img_uri, suffix)

            if retrieve_image_flag:
                req = urllib.Request(img_uri + suffix)
                try:
                    socket.setdefaulttimeout(__timeout__)
                    save_image(obj, req, img_path)
                    time.sleep(__sleep__)

                except urllib.HTTPError as e:
                    error(e)
                    img_path = ''
                    continue

        return os.path.basename(img_path)
    else:
        # for exclude fotolife
        img_path = img_src_dir + os.path.basename(img_uri)

        '''
        if check_local_img(img_path):
            return os.path.basename(img_path)
            '''

        if retrieve_image_flag:
            req = urllib.Request(img_uri)
            try:
                if not check_local_img(img_path):
                    socket.setdefaulttimeout(__timeout__)
                    save_image(obj, req, img_path)
                    time.sleep(__sleep__)

            except urllib.HTTPError as e:
                error(e)
                img_path = ''

        return os.path.basename(img_path)


def regex_search(pattern, string):
    """Prepare compilation of regex.

    Arguments:

        pattern: regex pattern
        string: processing target string

    return:

        pat_regex: compiled regex object
        match_obj: searching result object
    """
    pat_regex = re.compile(pattern, flags=re.U)
    match_obj = pat_regex.search(string)
    return pat_regex, match_obj


def error(e):
    """Print standard error

    Argument:

        e: Error message
    """
    sys.stderr.write("ERROR: %s\n" % e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests of __init__.py
"""
import unittest
import hatena2rest


class InitTests(unittest.TestCase):
    def test_version_defined(self):
        actual_version = hatena2rest.__version__
        self.assertTrue(actual_version)

    def test_sleep_defined(self):
        actual_sleep = hatena2rest.__sleep__
        self.assertTrue(actual_sleep)

    def test_timeout_defined(self):
        actual_timeout = hatena2rest.__timeout__
        self.assertTrue(actual_timeout)

    def test_log_defined(self):
        actual_log = hatena2rest.__log__
        self.assertTrue(actual_log)

    def test_dstdir_defined(self):
        actual_dstdir = hatena2rest.__dstdir__
        self.assertTrue(actual_dstdir)

    def test_imgdir_defined(self):
        actual_imgdir = hatena2rest.__imgdir__
        self.assertTrue(actual_imgdir)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import sys
from setuptools import setup, find_packages

sys.path.insert(0, 'src')
import hatena2rest

classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Information Technology",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Programming Language :: Python",
    "Topic :: Internet :: WWW/HTTP :: Site Management",
    "Topic :: Text Processing :: Markup",
    "Topic :: Text Processing :: Markup :: XML",
]

long_description = \
        open(os.path.join("docs","README.rst")).read() + \
        open(os.path.join("docs","HISTORY.rst")).read() + \
        open(os.path.join("docs","TODO.rst")).read()

requires = ['setuptools', 'sphinx', 'tinkerer']

setup(name='hatena2rest',
      version=hatena2rest.__version__,
      description='converting Hatena diary to reST format',
      long_description=long_description,
      author='Kouhei Maeda',
      author_email='mkouhei@palmtb.net',
      url='https://github.com/mkouhei/hatena2rest',
      license=' GNU General Public License version 3',
      classifiers=classifiers,
      packages=find_packages('src'),
      package_dir={'': 'src'},
      data_files = [],
      install_requires=requires,
      extras_require=dict(
        test=[
            'nose',
            'pep8',
            'unittest',
            ],
        ),
      test_suite='nose.collector',
      tests_require=['nose','pep8','unittest'],
      entry_points="""
        [console_scripts]
        htn2rst = hatena2rest.command:main
""",
)



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2012 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__version__ = '0.1'
__sleep__ = 1
__timeout__ = 4
__log__ = '/tmp/hatena2rest.log'
__dstdir__ = '~/tmp/hatena2rest/'
__imgdir__ = '/img/'
